"""Utility functions used by pyansiescapes.

Provides functions for either parsing arguments, getting correct ids from
ANSI-Enums, or checking for correct types, format etc.

"""

from pyansiescapes.enums import ANSICommands, TextAttributes, ColorDrawingLevel, Colors, Colors256
import pyansiescapes._types as t
from collections.abc import Iterable
import logging

_logger = logging.getLogger(__file__)
#--------------------- Parsers ------------------------------
def parsing_switcher(argc: int, arg: t.ColorArg) -> t.Callable[[t.ColorArg, int], t.Tuple[str, int]]:
    """Return get color_id function for argument arg.

    If argument is 'name', parse name first to check if user did not
    provide a hexadecimal, rgb or hsl value.

    Args:
        argc: The positional argument count [1-4] which specifies the argument type.

        arg: The color argument.

    Returns:
        The get_color_id_from_{type} function obj.
        Any of :func:`.get_color_id_from_id`, :func:`.get_color_id_from_name`,
        :func:`.get_color_id_from_hex` or
        :func:`.get_color_id_from_color_value`
    """
    switcher = {
        0: get_color_id_from_id,
        1: get_color_id_from_name,
        2: get_color_id_from_hex,
        3: lambda x, y: get_color_id_from_color_value(x, y, "rgb"),
        4: lambda x, y: get_color_id_from_color_value(x, y, "hsl"),
    }
    if argc == 1:
        # parse color name and then check again
        argc, arg = parse_color_name(arg)

    return switcher[argc]


def parse_drawing_level(drawing_level: t.DrawingLevelArg) -> str:
    """Parse drawing level into legal ANSI drawing level Sequence."""
    if isinstance(drawing_level, int):
        try:
            drawing_level = list(ColorDrawingLevel)[drawing_level]
        except IndexError:
            raise KeyError(
                "Cannot parse {} into type ColorDrawingLevel".format(drawing_level))

    elif not isinstance(drawing_level, ColorDrawingLevel):
        if drawing_level in ["3", "4"]:
            drawing_level = drawing_level
        else:
            try:
                drawing_level = ColorDrawingLevel[drawing_level]
            except KeyError:
                raise KeyError(
                    "Cannot parse {} into type ColorDrawingLevel".format(drawing_level))

    return drawing_level


def parse_colormode(colormode: int, blink: bool,
                    bright: bool, bold: bool) -> int:
    """Parse colormode setters into colormode.

    Returns the colormode in int. Blink and Bright will override colormode.
    Bold overrides colormode if :py:`colormode == 8`.

    Args:
        colormode: Colormode for colors. Any in :py:`[8, 16, 256]`.
        blink: Toggle blink/bright/256 bit mode.
        bright: Toggle blink/bright/256 bit mode.
        bold: Toggle bold/16-bit mode.

    Returns:
        Either 8, 16 or 256 (as integer) corresponding to the colormode.
    """
    if blink or bright or colormode == 256:
        colormode = 256
        return colormode
    elif bold or colormode == 16:
        colormode = 16
        return colormode
    else:
        return colormode


def parse_color_name(name: t.ColorArg) -> t.ColorArgTuple:
    """Parses the color name.

    Args:
        name: A color name. Can be any name in :class:`.Colors` or :class:`.Colors256` as :py:`str`.
            Strings with leading ``#`` will trigger hex value parsing.
            Tuple, list or array-like will trigger parsing as either rgb-
            or hsl-values based on the input values.
            See :func:`._parse_rgb_or_hsl` for details on parsing logic.

    Returns:
        A tuple where the first value defines the color argument category and
        the second value is the color argument parse into the correct type.

    Examples:
        >>> # Color name string as name argument
        >>> parse_color_name(name="blue")
        (1, 'blue')

        >>> # Hexadecimal color value as "name" argument
        >>> parse_color_name(name="#ffffff")
        (2, '#ffffff')

        >>> # Rgb-values as "name" argument
        >>> parse_color_name((255, 0, 0))
        (3, (255, 0, 0))

        >>> # Color-id as "name" argument
        >>> parse_color_name(name=1)
        (0, 1)

        >>> # Color-id string as "name" argument
        >>> parse_color_name(name="1")
        (0, 1)

        >>> # Received incorrect argument for "name". Raise TypeError.
        >>> parse_color_name(name=None)
        Traceback (most recent call last):
        ...
        TypeError: Cannot understand \"name=None\" input argument type

    """
    try:
        name = int(name)
    except (ValueError, TypeError):
        pass
    # if name is and integer, its a color_id
    if isinstance(name, int):
        return 0, name
    elif not isinstance(name, str):
        # user provided rgb or hsl value
        if isinstance(name, Iterable):
            return 3, name
        else:
            # cannot understand input argument type
            raise TypeError(
                "Cannot understand \"name={}\" input argument type".format(name))
    elif name.startswith('#'):
        # user input name is a hex colorvalue
        return 2, name

    return 1, name


def parse_hex(hex: str) -> str:
    """Checks if hex is valid and return it in Colors256-key format.

    Args:
        hex: Hexadecimal color value. Must start with ``#``, be of length 7 and
            must be a valid hexadecimal value (all characters must be in
            :py:`set("#abcdef0123456789")`).

    Returns:
        The hexadecimal color value in Colors256-key-format (eg.
        :py:`"hex_ffffff"`).

    Raises:
        TypeError: {hex} is not a valid hexadecimal color value
            Raised if the input hex string is not valid.

    Examples:
        >>> # Parse white hexadecimal color value into Colors256-key.
        >>> parse_hex("#ffffff")
        'hex_ffffff'

        >>> # Raise TypeError with hexadecimal color value is wrong.
        >>> parse_hex("ffffff")
        Traceback (most recent call last):
        ...
        TypeError: \"ffffff\" is not a valid hexadecimal color value

        >>> # Raise TypeError with hexadecimal color value is wrong.
        >>> parse_hex("#gfffff")
        Traceback (most recent call last):
        ...
        TypeError: \"#gfffff\" is not a valid hexadecimal color value

    """
    if is_valid_hex_string(hex):
        hex = 'hex_' + hex[1:]
    else:
        raise TypeError(
            "\"{}\" is not a valid hexadecimal color value".format(hex))

    return hex


def parse_color_value(color_value: t.ColorValue, key: str) -> str:
    """Parses color value to color key

    RGB-values will be cliped to their respective closest bin.
    """
    if key == "rgb":
        color_value = clip_to_closes_color(color)
    return key + '_'.join((str(val) for val in rgb))


def parse_arguments(*args: t.Any) -> t.Iterator[t.ColorArgTuple]:
    """Yields arguments that are not None."""
    for argc, arg in enumerate(args):
        if arg is not None:
            yield (argc, arg)


def clip_to_closes_color(color: t.ColorValue) -> t.Iterator[int]:
    """Clips the rgb values to their respective closest bin."""
    assert isinstance(color, Iterable)

    bins = [0, 95, 135, 175, 215, 255]
    color = (_round_to_next_bin(x, bins) for x in color)
    return color


def _round_to_next_bin(val: int, bins: t.ColorBins) -> int:
    """Return the bin in bins closest to value."""
    right = bisect_right(bins, val)
    left = right - 1
    ret = _get_closest(val, *bins[left:right+ 1]
                       ) if right != len(bins) else bins[left]
    return ret


# ---------------------- Getters ---------------------------------------
def _get_closest(val: int, left: int, right: int) -> int:
    """Return left or right depending on which is closer to val"""
    ret = left if val - left < right - val else right
    return ret


def get_first_color_argument(*args: t.Any) -> t.ColorArgTuple:
    """Return first argument in args which is not None.

    Raises:
        TypeError: Cannot parse ``None`` into a color. Please provide a color!
            If all arguments are ``None``.
    """
    try:
        argc, arg = next(parse_arguments(*args))
    except StopIteration:
        raise TypeError(
            "Cannot parse 'None' into a color. Please provide a color!")

    return argc, arg


def get_color_id_from_id(color_id: int, colormode: int) -> t.Tuple[str, int]:
    """Return color_id as str and toggle colormode 256 if necessary."""
    if color_id > 8:
        colormode = 256
    return str(color_id), colormode


def get_color_id_from_color_enum(key: str, color_enum: t.ColorEnum) -> str:
    """Return color_id as str for key in color_enum

    Args:
        key: A Color key as str.
            Any of format:

                - name: :py:`"^[a-z, A-z, 0-9]*"` (e.g. :py:`"blue"`)
                - hex: :py:`"^hex_[0-9,a-f]{6}$"` (e.g. :py:`"hex_ffffff"`)
                - rgb: :py:`"^rgb_[0-9]{1,3}_[0-9]{1,3}_[0-9]{1,3}$"`
                  (e.g. :py:`"rbg_255_0_0"`)
                - hsl: :py:`"^hsl_[0-9]{1,3}_[0-9]{1,3}_[0-9]{1,3}$"`
                  (e.g. :py:`"hsl_255_0_0"`)

        color_enum: An enum-obj of type :class:`.Colors` or :class:`.Colors256`.

    Returns:
        A color id as str (:py:`"^[0-9]{1,3}$"` e.g. :py:`"102"`).

    Raises:
        KeyError: {key} is not a valid color key!
            If key is not a valid color key for Color Enum.
    """
    try:
        color_id = color_enum[key]
    except KeyError:
        raise KeyError("{} is not a valid color key for {}!".format(key, color_enum))
    return color_id


def get_color_id_from_name(name: str, colormode: int) -> t.Tuple[str, int]:
    """Returns a color id and the correct colormode.

    Parses name into all lower cases letters and toggles colormode 256 if
    name is not an 8-bit Color name.

    Args:
        name: A color name.
            Format:

                - :py:`'^[a-z, A-z, 0-9]*'` (e.g. :py:`"blue"`).

        colormode: Any integer in :py:`[8, 16, 256]`.
    """
    if colormode == 256:
        color_enum = Colors256
    else: # colormode != 256
        if not is_8bit_color(name):
            colormode = 256
            color_enum = Colors256
        else:
            color_enum = Colors

    return get_color_id_from_color_enum(name.lower(), color_enum), colormode


def get_color_id_from_hex(hex: str, colormode: int) -> t.Tuple[str, int]:
    """Returns a color id and the correct colormode.

    Parses hex str (e.g. :py:`"#ffffff"`) into valid key format (e.g. :py:`"hex_ffffff"`).

    Args:
        hex: A hexadecimal color value.
            Format:

                - :py:`"^hex_[0-9,a-f]{6}$"` (e.g. :py:`"hex_ffffff"`)

        colormode: Just provided for compatibility but is ignored since
            hexdecimalcolor value toogles colormode 256 automatically.
    """
    key = parse_hex(hex)
    return get_color_id_from_color_enum(key, Colors256), 256


def get_color_id_from_color_value(color_value: t.ColorValue,
                                  colormode: int) -> t.Tuple[str, int]:
    """Returns a color id and the correct colormode.

    Iterarble (eg. :py:`(255, 0, 0)` are parsed into valid key format (eg.
    :py:`"rgb_255_0_0"`) before lookup. See :func:`.parse_color_value` for
    details.

    Args:
        color_value: A color value as :py:`list` or :py:`tuple` with
            :py:`len(color_value) == 3`

        colormode: Just provided for compatibility but is ignored since rgb/hsl
            color values toogle colormode 256 automatically.
    """
    key = parse_color_value(color_value)
    return get_color_id_from_color_enum(key, Colors256)


def get_color_string(color_id: str, colormode: int) -> str:
    """Return get_color_string func for colormode."""
    switcher = {8: get_color_string_8_bit,
                16: get_color_string_16_bit,
                256: get_color_string_256_bit
                }
    return switcher[colormode](color_id)


def get_color_string_256_bit(color_id: str) -> str:
    """Return ANSI color command string for 256 bit colors"""
    return Colors._blink + ANSICommands.separator + \
        TextAttributes.blink + ANSICommands.separator + color_id


def get_color_string_16_bit(color_id: str) -> str:
    """Return ANSI color command string for 16 bit (bold) colors"""
    return color_id + ANSICommands.separator + TextAttributes.bold


def get_color_string_8_bit(color_id: str) -> str:
    """Return ANSI color command string for 8 bit colors"""
    return color_id


#--------------------- Checker ------------------------------
def any_is_not_none(*args: t.Any) -> bool:
    return any(arg is not None for arg in args)


def all_are_none(*args: t.Any) -> bool:
    return all(arg is None for arg in args)


def any_is_none(*args: t.Any) -> bool:
    return any(arg is None for arg in args)


def all_are_not_none(*args: t.Any) -> bool:
    return all(arg is not None for arg in args)


def is_8bit_color(name: str) -> bool:
    """Return True if name is a valid key of Colors-Enum."""
    return name in Colors.__members__.keys()


def is_valid_hex_string(hex: str) -> bool:
    """Return True if hex is a valid hexadecimal color value string."""
    return (hex.startswith("#") and len(hex) ==
            7 and has_only_valid_characters(hex))


def has_only_valid_characters(hex: str) -> bool:
    """Return True if hex has only valid hexadecimal characters."""
    _valid_hex_characters = set("#abcdef0123456789")
    return set(hex) <= _valid_hex_characters


if __name__ == '__main__':
    import doctest
    doctest.testmod()



"""API for console manipulation using ANSI Escape sequences in Python.
"""
import logging
from itertools import chain
from pyansiescapes.enums import ANSICommands, TextAttributes, ColorDrawingLevel, Colors
from pyansiescapes import utils
import pyansiescapes._types as t
import emojis

_logger = logging.getLogger(__file__)


# Curser controls:
def cursor_up(number_of_lines: int = 1) -> str:
    """Moves cursor up number_of_lines lines."""
    return ANSICommands.start + "{:d}A".format(number_of_lines)


def cursor_down(number_of_lines: int = 1) -> str:
    """Moves cursor down number_of_lines lines."""
    return ANSICommands.start + "{:d}B".format(number_of_lines)


# Clearers:
def clear_to_end_of_line() -> str:
    """Clears line from current cursor position to the end."""
    return ANSICommands.start + "0K"


def clear_to_start_of_line() -> str:
    """Clears line from start to current cursor position."""
    return ANSICommands.start + "1K"


def clear_line() -> str:
    """Clears the entire line from start to end."""
    return ANSICommands.start + "2K"


def clear_lines(number_of_lines: int = 1) -> str:
    """Clears number_of_lines lines."""
    return (clear_line() + cursor_up()) * number_of_lines


def clear_screen_until_end() -> str:
    """Clears screen from current cursor position to the end."""
    return ANSICommands.start + "0J"


def clear_screen_to_beginning() -> str:
    """Clears screen from current cursor position to the beginning."""
    return ANSICommands.start + "1J"


def clear_screen() -> str:
    """Clears the entire screen. """
    return ANSICommands.start + "2J"


# Rich Text formatting:

def format(text: str, *args: t.Any, **kwargs: t.Any) -> str: # pylint: disable=redefined-builtin
    """Return formatted text as str.

    Uses the formatting attributes given in `*args` or `**kwargs`.
    Postional arguments or keywordarguments that are not supported
    will be ignored.

    Args:
        text: The text/string that should be formatted.
        *args: Any number of postional arguments. Unsupported arguments will be
            ignored. See "supported arguments" section for futher details.
        **kwargs: Any number of keyword arguments. Unsupported keyword arguments
            will be ignored. See "supported keywords" section for further
            details.

    Returns:
        The text with leading ANSI Escape sequence "rich text"
        commands and a trailing ANSI Escape sequence "reset"
        command.

    Examples:
        >>> # Format text to bold and underlined using postional arguments
        >>> format('Hello ANSI!', 'bold', 'underline')
        \'\\x1b[1;4mHello ANSI!\\x1b[0m\'

        >>> # It's also possible to use keyword arguments with True, False
        >>> format('Hello ANSI!', bold=True, underline=True)
        \'\\x1b[1;4mHello ANSI!\\x1b[0m\'

        >>> # You can also set font and background color
        >>> format('Hello ANSI!', color='blue', background='white')
        \'\\x1b[34;47mHello ANSI!\\x1b[0m\'

        >>> # Or provide a keyword argument dict
        >>> format('Hello ANSI!', color={'name':'blue', 'colormode':256}, background='white')
        \'\\x1b[38;5;12;47mHello ANSI!\\x1b[0m\'
    """
    # parse positional text attributes
    text_attribute_arguments = (arg for arg in args
                                if arg in TextAttributes.__members__.keys()) # pylint: disable=no-member
    # parse keyword text attribute arguments:
    text_attribute_keywords = (
        kwarg for kwarg, value in kwargs.items()
        if value and kwarg in TextAttributes.__members__.keys()) # pylint: disable=no-member
    text_attributes = (
        TextAttributes[key]
        for key in chain(text_attribute_arguments, text_attribute_keywords))
    # parse positional
    color_attributes = []
    for i, key in enumerate(["color", "background"]):
        try:
            value = kwargs[key]
        except KeyError:
            continue
        if isinstance(value, (tuple, list)):
            attribute = _color(*value, drawing_level=i)
        elif isinstance(value, dict):
            attribute = _color(**value, drawing_level=i)
        else:
            attribute = _color(value, drawing_level=i)

        color_attributes.append(attribute)

    # encode emojis:
    text = emojis.encode(text)

    return _format_rich_text(*text_attributes,
                             *color_attributes) + text + reset()


# TextAttributes:
def reset() -> str:
    """Returns 'reset' ANSI-command string."""
    return _format_rich_text(TextAttributes.reset)


def bold() -> str:
    """Returns 'bold text' ANSI-command string."""
    return _format_rich_text(TextAttributes.bold)


def underscore() -> str:
    """Returns 'underlined/underscored text' ANSI-command string."""
    return _format_rich_text(TextAttributes.underscore)


def underline() -> str:
    """Returns 'underlined/underscored text' ANSI-command string."""
    return _format_rich_text(TextAttributes.underline)


def bright() -> str:
    """Returns 'bright/blink text' ANSI-command string."""
    return _format_rich_text(TextAttributes.bright)


def blink() -> str:
    """Returns 'bright/blink text' ANSI-command string."""
    return _format_rich_text(TextAttributes.blink)


def reversed() -> str: # pylint: disable=redefined-builtin
    """Returns 'reversed text' ANSI-command string."""
    return _format_rich_text(TextAttributes.reversed)


def concealed() -> str:
    """Returns 'concealed text' ANSI-command string."""
    return _format_rich_text(TextAttributes.concealed)


# Colors:
def color(*args: t.Any, **kwargs: t.Any) -> str:
    """Returns ANSI Escape Sequence for specified color.

    Args:
        *args: any valid positional argument for :func:`._color`.
        **kwargs: any valid keyword argument for :func:`._color`.

    Examples:
        >>> # Get string for 8-bit red text coloring (e.g. foreground).
        >>> color(name="red")
        \'\\x1b[31m\'

        >>> # Get string for bold red background.
        >>> color(name="red", bold=True, drawing_level="background")
        \'\\x1b[41;1m\'

        >>> # Get string for 256-bit red background.
        >>> color(name="red", bold=True, blink=True, drawing_level="background")
        \'\\x1b[48;5;9m\'

        >>> # Get string for mediumspringgreen foreground 256-bit color.
        >>> color(name="mediumspringgreen")
        \'\\x1b[38;5;49m\'

    """
    return _format_rich_text(_color(*args, **kwargs))


def _color(name: t.Optional[t.ColorArg] = None, # pylint: disable=too-many-arguments
           color_id: t.Optional[t.Union[int, str]] = None,
           hexa: t.Optional[str] = None,
           rgb: t.Optional[t.ColorValue] = None,
           hsl: t.Optional[t.ColorValue] = None,
           drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground,
           bold: bool = False, # pylint: disable=redefined-outer-name
           blink: bool = False, # pylint: disable=redefined-outer-name
           bright: bool = False, # pylint: disable=redefined-outer-name
           colormode: int = 8) -> str:
    """Returns ANSI color-string for specified color.

    Color value argument get parsed in this order:

    - \(0) colorid, (1) name, (2) hexa, (3) rgb, (4) hsl.

    Colormode gets toggled in this order:

    - Conditions for 256-bit colormode are checked first:
        - :py:`color_id is not None and color_id > 8`
        - :py:`name not in` :class:`.Colors` or ``name`` is in hexa-, rgb- or
          hsl-format (see below)
        - :py:`any(hexa, rgb, hsl)`
        - :py:`bright or blink or colormode == 256`

    - Next the condition for 16-bit colormode are checked:
        - :py:`((color_id < 8 or name in` :class:`.Colors` :py:`) and bold)`
          :py:`or colormode == 16`

    - If none of the above conditions are true, fallback to default
        8-bit colormode.

    Args:
        name: A color name.
            Can be of format:

            - Any name in :class:`.Colors` or :class:`.Colors256`.
            - Integer or Integer-string (triggers color id parsing.)
                - Strings with leading ``#`` (triggers hex-value parsing.)
            - Tuple, list or array-like (triggers parsing as either rgb- or
              hsl-values based on the input values.)

            See :func:`.utils.parse_color_name` for details on parsing logic.
        color_id: The color id as integer or integer-string.
        hexa: A hexadecimal color value as str. Must start with a leading ``#``.

            See :func:`.utils.parse_hex` for details on parsing logic.
        rgb: Color values in rgb color space. Must be iterable and of length 3.
            See :func:`.utils.parse_color_value` for details on parsing logic.
        hsl: Color values in hsl color space. Must be iterable and of lenght 3.
            See :func:`.utils.parse_color_value` for detail on parsing logic.
        drawing_level:
            The color drawing level.

            Valid foreground values are:
                - :py:`'foreground'`, :attr:`.ColorDrawingLevel.foreground`,
                  :py:`0`, :py:`False`, :py:`'3'`

            Valid background values are:
                - ``'background'``, :attr:`.ColorDrawingLevel.background`,
                  :py:`1`, :py:`True`, :py:`'4'`

            Default: :py:`"foreground"`
        bold: Triggers bold colors (16-bit).
            Default: False
        blink: Triggers "blink/birght" colors (256-bit).
            Deafault: False
        bright: Triggers "blink/birght" colors (256-bit).
            Default: False
        colormode: Triggers 8-, 16-, or 256-bit colors. Any in (8, 16, 256).
            Default: 8

    Raises:
        TypeError: If all color arguments are None.
    """
    # Parse drawing level
    drawing_level = utils.parse_drawing_level(drawing_level)
    colormode = utils.parse_colormode(colormode, blink, bright, bold)
    _logger.debug("%d", colormode)
    # Check if a valid color was provided
    argc, arg = utils.get_first_color_argument(color_id, name, hexa, rgb, hsl)
    # Look-up correct get color id function
    get_color_id = utils.parsing_switcher(argc, arg)
    color_id, colormode = get_color_id(arg, colormode)
    _logger.debug("%s, %d", color_id, colormode)
    color_string = utils.get_color_string(color_id, colormode)

    return drawing_level + color_string


# convenience functions:
def color_8bit(
        name: str,
        drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground
) -> str:
    """Convenience function for 8-bit colors.

    Trys to find the color_id for provided name. Returns an empty string if name
    is not an 8-bit color.

    Args:
        name: A color name. Any color name in Colors.
        drawing_level: The color drawing level.

            Valid foreground values are:
                - "foreground", :attr:`.ColorDrawingLevel.foreground`, 0, "3"

            Valid background values are:
                - "background", :attr:`.ColorDrawingLevel.background`, 1, "4"

            Default: "foreground"

    """
    if utils.is_8bit_color(name):
        drawing_level = utils.parse_drawing_level(drawing_level)
        return _format_rich_text(drawing_level + Colors[name])
    return ""


def background(*args: t.Any, **kwargs: t.Any) -> str:
    """Convenience function for colored backgrounds.

    Returns a call to :func:`.color` with drawing_level= :attr:`.ColorDrawingLevel.background`.
    See color for further description of input arguments. Drawing_level keyword
    argument should be omitted."""
    return color(*args, **kwargs, drawing_level=ColorDrawingLevel.background)


def black(
        drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground
) -> str:
    """Convenience function for 8-bit black color.

	See :func:`.color_8bit` for further details.
	"""
    return color_8bit("black", drawing_level)


def red(drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground
        ) -> str:
    """Convenience function for 8-bit red color.

	See :func:`.color_8bit` for further details.
	"""
    return color_8bit("red", drawing_level)


def green(
        drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground
) -> str:
    """Convenience function for 8-bit green color.

	See :func:`.color_8bit` for further details.
	"""
    return color_8bit("green", drawing_level)


def yellow(
        drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground
) -> str:
    """Convenience function for 8-bit yellow color.

	See :func:`.color_8bit` for further details.
	"""
    return color_8bit("yellow", drawing_level)


def blue(
        drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground
) -> str:
    """Convenience function for 8-bit blue color.

	See :func:`.color_8bit` for further details.
	"""
    return color_8bit("blue", drawing_level)


def magenta(
        drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground
) -> str:
    """Convenience function for 8-bit magenta color.

	See :func:`.color_8bit` for further details.
	"""
    return color_8bit("magenta", drawing_level)


def cyan(
        drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground
) -> str:
    """Convenience function for 8-bit cyan color.

	See :func:`.color_8bit` for further details.
	"""
    return color_8bit("cyan", drawing_level)


def white(
        drawing_level: t.DrawingLevelArg = ColorDrawingLevel.foreground
) -> str:
    """Convenience function for 8-bit white color.

	See :func:`.color_8bit` for further details.
	"""
    return color_8bit("white", drawing_level)


def _format_rich_text(*formatting_commands: str) -> str:
    """Return an ANSI Escapes Sequence for rich text formatting.

    Mutiple formatting commands are chained using the ANSI command seperator.
    """
    ansi_command = ANSICommands.start
    logging.debug("_format_rich_text received: %s",
                  (", ".join(formatting_commands)))
    ansi_command += ANSICommands.separator.join(formatting_commands)
    ansi_command += ANSICommands.stop
    logging.debug("%s", ANSICommands._debug_esc + ansi_command[1:]) # pylint: disable=protected-access
    return ansi_command


if __name__ == '__main__':
    import doctest
    doctest.testmod()



"""Types and type aliases used within the pyansiescapes package.

Types declared here:
    - **Colorbins**: a list of int
    - **ColorValueTuple**: integer tuple of size 3 (for rgb, hsl values)
    - **ColorValueList**: list of int (should be of size 3 for rgb hsl)
    - **ColorValue**: accept either ColorValueTuple or ColorValueList
    - **ColorArg**: all types accepted by :func:`.color`
        - int --> color id (e.g. 12)
        - str --> color id as str ("12"), name ("blue"), hexa ("#0000ff)
        - Colors --> obj of type :class:`.Colors`
        - Colors256 --> obj of type :class:`.Colors256`
        - ColorValue --> see **ColorValue**
    
    - **ColorArgTuple**: Tuple[int, ColorArg] used internally for parsing:
        - int: any in range(5)
            - 0 --> color argument is color id (e.g. 12 or "12")
            - 1 --> color argument is name (e.g. "blue")
            - 2 --> color argument is hexa (e.g. "#0000ff)
            - 3 --> color argument is rgb-value (e.g. (0, 0, 255))
            - 4 --> color argument is hsl-value (e.g. (240, 100, 50))
    
    - **ColorEnum**: either :class:`.Colors` or :class:`.Colors256`
    - **DrawingLevelArg**: type for *drawing_level* arg in :func:`._color`
        - int --> e.g. 3
        - bool --> e.g. False/0
        - str --> "foreground"
        - ColorDrawingLevel --> see :class:`.ColorDrawingLevel`
"""
from typing import List, Dict, Tuple, Callable, Any, Optional, Iterator, Union
from pyansiescapes.enums import *

ColorBins = List[int]
ColorValueTuple = Tuple[int, int, int]
ColorValueList = List[int]
ColorValue = Union[ColorValueTuple, ColorValueList]
ColorArg = Union[int, Colors, Colors256, str, ColorValue]
ColorArgTuple = Tuple[int, ColorArg]
ColorEnum = Union[Colors, Colors256]
DrawingLevelArg = Union[int, bool, str, ColorDrawingLevel]



from setuptools import setup, find_packages
import os
from version_query import predict_version_str

file_dir = os.path.abspath(os.path.dirname(__file__))
__version__ = predict_version_str().split("+") [0]
name = 'pyansiescapes'

# Get the long description from the README file
with open(os.path.join(file_dir, 'README.md'), encoding = 'utf-8') as f:
    long_description = f.read()

setup(
    name = name,
    version = __version__,
    author='Till Hainbach',
    author_email='Till.Hainbach@posteo.de',
    description = 'Python package for quick and readable console manipulation using ANSI Escapes Sequences',
    long_description = long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/tillhainbach/' + name,
    packages = find_packages(),
    install_requires=[
          'emojis',
      ],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3 ',
        'Programming Language :: Python :: 3 :: Only',
    ],
    python_requires='>=3.5',
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/tillhainbach/' + name + '/issues',
        'Documentation': 'https://pyansiescapes.readthedocs.io/en/latest/',
        'Source': 'https://github.com/tillhainbach/' + name,
    },
)
